/*!!! This is the beginning of the C version of sockz !!! */

/*                   __           Stack Oriented Calculus Kernel in Z  */
/* _____ _____ _____|  |__ _____      copyright Alain Brobecker        */
/*|__  -|  _  |  ___|    _|-  __|                                      */
/*|_____|_____|_____|__|__|_____| v2.0, 2004feb23                      */

#include <stdio.h>
/**** platform dependant definitions ****/
typedef unsigned int uint32; /* must be 32 bits unsigned integer */

/**** external functions ****/
extern int exit(int);
extern int calloc(unsigned int,unsigned int);
extern int strlen(char *);
extern int strcpy(char *,char *);
extern int strcat(char *,char *);

/**** global variables ****/
int a,b,c;
FILE *filep,*libp,*fopen(const char *,const char *);
char *filename, *src;
long int srclen=0;
unsigned int memsize=128*256; /* number of uint32 */
uint32 *mem;

/*****************************/
/*                           */
/*  error handling routines  */
/*                           */
/*****************************/
void error_usage() {
  printf("usage: sockz filename [-l*] [-s* number]\n");
  printf("   or: sockz \"sockz expression\" [-l*] [-s* number]\n");
  exit(1);
}
void error_memory() {
  printf("error: not enough memory\n");
  exit(1);
}

/***************************************/
/*                                     */
/*  main code: tokeniser and executer  */
/*                                     */
/***************************************/
int main(argc,argv) int argc; char *argv[]; {
  uint32 lib=0, file=0;

  /**** Parse command line for arguments ****/
  if(argc<2) { error_usage(); }
  for(a=1;a<argc;a++) {
    if(argv[a][0]=='-') { /* argument is a swicth, -lib or -size */
      switch(argv[a][1]) {
        case 'l': lib=1; break;     /* -lib switch */
        case 's':                   /* -size switch */
          memsize=0;
          if(a++>=argc) { error_usage(); } /* next arg */
          for(b=0;argv[a][b]!='\0';b++) { /* convert next arg as number */
            c=(int) (argv[a][b]-'0');
            if( (c<0) || (c>9)) { error_usage(); }
            memsize=memsize*10 + (uint32) c;
          }
          memsize=memsize*256; /* number of uint32 */
          if(memsize==0) { error_memory(); };
          break;
        default: error_usage();/*!!! break;*/
      }
    } else { /* argument is a filename or a sockz expression */
      if(file!=0) { error_usage(); } /* two or more file args? */
      file=1; /* 1 for file, 2 for expression */
      filename=argv[a];
      for(b=0;filename[b]!='\0';b++) { /* contains spaces? => expression */
        if(filename[b]==' ') { file=2; }
      }
    }
  }

  /**** Set memory, load file or copy expression, and load library ****/
  mem=(uint32 *) calloc(memsize,4); /* memory for znumbers */
  /* check if library is needed, and set srclen accordingly */
  if(lib==1) {
    libp=fopen("lib","rb");
    if(libp==0) { printf("error: \"lib\" file not found\n"); exit(1); }
    fseek(libp,0,2); /* 2 for EOF */
    srclen=ftell(libp)+6; /* +6 to add " QUIT " */
    fseek(libp,0,0); /* 0 for SOF */
  }
  /* allocate memory & put file/expression in src */
  switch(file) {
    case 0: error_usage();/*!!! break;*/ /* neither file nor expression */
    case 1: /**** file ****/
      filep=fopen(filename,"rb");
      if(filep==0) { printf("error: %s not found\n",filename); exit(1); }
      fseek(filep,0,2); /* 2 for EOF */
      srclen=srclen+ftell(filep)+7; /* +7 to add " QUIT \0" */
      src=(char *) calloc((unsigned int) srclen,1); /* memory for src */
      fseek(filep,0,0); /* 0 for SOF */
      a=0;
      while((c=getc(filep))!=EOF) { src[a++]=c; }
      break;
    case 2: /**** expression ****/
      a=strlen(filename);
      srclen=srclen+a+7; /* +7 to add " QUIT \0" */
      src=(char *) calloc((unsigned int) srclen,1); /* memory for src */
      strcpy(src,filename); /* copy expression */
      break;
  }
  src[a]='\0'; /* \0 necessary for strcat */
  strcat(src," QUIT "); a=a+6; /* append " QUIT " */
  /* append library+" QUIT " if needed */
  if(lib==1) {
    while((c=getc(libp))!=EOF) { src[a++]=c; }
    src[a]='\0'; /* \0 necessary for strcat */
    strcat(src," QUIT "); a=a+6; /* append " QUIT " */
  }
  src[a]='\0'; /* null terminator */

  printf("%s\n",src); /* !!! */
}
